package Controladores;

import java.util.Scanner;
import Model.Jugadores.*;
import Model.Movimientos.*;
import Model.Reglas.*;
import Model.Enumerate.*;
import Model.Exceptions.*;
import Model.Partida;
import Controladores.Factorias.*;

public class Controlador {
	
	private Scanner sc;
	private Jugador jugador1;
	private Jugador jugador2;
	private FactoriaJuegos factoria;
	private ReglasJuego reglas;
	private Partida partida;
	private boolean salir;
	
	//Constructor
	public Controlador(FactoriaJuegos factoria, Partida partida, Scanner sc){
		
		this.factoria = factoria;
		this.partida = partida;
		this.sc = sc;
		this.salir = false;
		//Por defecto se ponen ambos jugadores como jugadores humanos
		this.jugador1 = factoria.creaJugadorHumano(sc);
		this.jugador2 = factoria.creaJugadorHumano(sc);
		
	}
	
	//Metodos
	public ReglasJuego getReglasjuego(){
		return this.reglas;
		
	}
	
	public void setReglasJuego(ReglasJuego reglas){
		this.reglas = reglas;
		
	}
	
	public void setFactoria(FactoriaJuegos factoria){
		
		this.factoria = factoria;
		//this.jugador1 = factoria.creaJugadorHumano(sc);
		//this.jugador2 = factoria.creaJugadorHumano(sc);
		
	}
	
	public void reset(FactoriaJuegos factoria){

		//this.setFactoria(factoria);
		this.partida.reset(factoria.creaReglas());
		this.jugador1 = factoria.creaJugadorHumano(sc);
		this.jugador2 = factoria.creaJugadorHumano(sc);
		
	}
	
	public void jugar(FactoriaJuegos factoria) throws ErrorDeEjecucion {
		
		if(factoria != null){
			this.reset(factoria);
		}
		else{
			throw new ErrorDeEjecucion("El tipo de juego es incorrecto.");
		}
		
	}
	
	public void ponerJugador(Ficha turno, String tipoJugador) throws ErrorDeEjecucion {
		
		if(turno == Ficha.BLANCA){
			if(tipoJugador.equalsIgnoreCase("humano")){
				this.jugador1 = this.factoria.creaJugadorHumano(sc);
			}
			else if(tipoJugador.equalsIgnoreCase("aleatorio")){
				this.jugador1 = this.factoria.creaJugadorAleatorio();
			}
			else{
				throw new ErrorDeEjecucion("El tipo de jugador es incorrecto.");
			}
		}
		else if(turno == Ficha.NEGRA){
			if(tipoJugador.equalsIgnoreCase("humano")){
				this.jugador2 = this.factoria.creaJugadorHumano(sc);
			}
			else if(tipoJugador.equalsIgnoreCase("aleatorio")){
				this.jugador2 = this.factoria.creaJugadorAleatorio();
			}
			else{
				throw new ErrorDeEjecucion("El tipo de jugador es incorrecto.");
			}
		}
		else{
			throw new ErrorDeEjecucion("El color introducido es incorrecto.");
		}
		
	}
	
	private Jugador getjugador(){
		
		Jugador jugadorActual;
		if(this.partida.getTurno() == Ficha.BLANCA){
			jugadorActual = jugador1;
		}
		else{
			jugadorActual = jugador2;
		}
		return jugadorActual;
		
	}	
	
	public void poner() {
		
		//boolean finPartida = false;
		//Movimiento mv = new MovimientoConecta4(columna,this.partida.getTurno());
		try{
			Movimiento mv = this.partida.getMovimiento(factoria, this.getjugador());
			this.partida.ejecutaMovimiento(mv);
			if (this.partida.partidaTerminada()) {
				if (this.partida.getGanador() != Ficha.VACIA) {
					System.out.print(System.getProperty("line.separator"));
					System.out.print(this.partida.toString());
					System.out.print(System.getProperty("line.separator"));
					System.out.print("Ganan las: "+ this.partida.escribeGanador());
					System.out.print(System.getProperty("line.separator"));
				} 
				else {// La partida ha acabado en tablas
					System.out.print(System.getProperty("line.separator"));
					System.out.print(this.partida.toString());
					System.out.print(System.getProperty("line.separator"));
					System.out.println("La partida acabo en tablas.");
					System.out.print(System.getProperty("line.separator"));
				}
				this.salir = true;
			}
		}
		catch(DatosIncorrectos e){
			System.out.println(e.getMessage());
		}
		catch(MovimientoInvalido e){
			System.out.println(e.getMessage());
		}
		
	}
	
	public void undo() throws ErrorDeEjecucion{
		
		this.partida.undo();
		
	}
	
	public void salir(){
		
		System.out.println("Has salido del juego.");
		System.out.print(System.getProperty("line.separator"));
		System.out.println("HASTA PRONTO!!!");
		System.out.println(System.getProperty("line.separator"));
		this.salir = true;
	}
	
	private String leerLineaDeComando() {
		
		System.out.println("Introduzca su opcion: ");
		String linea = sc.nextLine();
		return linea;

	}
	
	private void menu(){
		
		System.out.print(System.getProperty("line.separator"));
		System.out.println(this.partida.toString());
		System.out.println(this.partida.indicarTurno());
		this.partida.escribeGanador();
		//Leemos el comando que metemos en la entrada
		String datos = this.leerLineaDeComando();
		//Tratamos los datos introducidos por linea de comandos
		try{
			Comandos comando = ParserAyudaComandos.parseaComando(datos);
			comando.execute(this);
		}
		catch(ErrorDeEjecucion e){
			System.out.println(e.getMessage());
		}
		
	}
	
	public void run(){
		
		System.out.println("Bienvenido al juego del Conecta 4.");
		System.out.println("==================================");
		
		while(!salir){
			this.menu();
			
		}
	}

}
