package Main;

import org.apache.commons.cli.BasicParser;
import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;

import java.util.Scanner;

import Model.Enumerate.*;
import Controladores.Factorias.*;
import Model.Reglas.*;
import Model.Partida;
import Controladores.Controlador;

public class Main {
	
	//Atributos
	private static final String GET_HELP = "\nUse -h|--help para mas detalle.";
	private static final String ERR_PARAM = "Uso incorrecto: ";
	private static final String NO_GAME = "Uso incorrecto: Juego ";
	private static final String MORE_PARAM = "Uso incorrecto: Argumentos no entendidos: ";
	
	private static class SetupConfigurator{
		
		private static TipoJuego juego;
		private static String numFilas;
		private static String numColumnas;
		
		//Definimos una clase interna.
		public SetupConfigurator(TipoJuego juego, String numFilas, String numColumnas){
			
			SetupConfigurator.juego = juego;
			SetupConfigurator.numFilas = numFilas;
			SetupConfigurator.numColumnas = numColumnas;
			
		}
		
		public TipoJuego getJuego(){
			
			return SetupConfigurator.juego;
			
		}
		
		public String getNumFilas(){
			
			return SetupConfigurator.numFilas;
			
		}
		
		public String getNumColumnas(){
			
			return SetupConfigurator.numColumnas;
			
		}
		
		
	}//Fin de la clase interna
	
	private static void showHelp(){
		
		System.out.println("usage: Main [-g <game> [h] [-x <rowNumber>] [-y <columnNumber>]");
		System.out.println("-g --game <game> Tipo de juego(c4, co, gr). Por defecto c4.");
		System.out.println("-h --help Muestra la ayuda sobre como utilizar los parámetros.");
		System.out.println("-y --tamY <columnNumber> Muestra el numero de columnas solo en el juego de Gravity."
								+ " Por defecto es 10.");
		System.out.println("-x --tamX <rowNumber> Muestra el numero de filas solo en el juego de Gravity."
				                + "Por defecto es 10.");
		
	}
	
	private static SetupConfigurator setupMode(CommandLine line, TipoJuego juego){
		
		String filas = "";
		String columnas = "";
		if(line.hasOption("y")){
			if(juego == TipoJuego.GRAVITY){
				columnas = line.getOptionValue("y");
			}
			else{
				System.err.println(MORE_PARAM);
				System.exit(1);
			}
		}
		if(line.hasOption("x")){
			if(juego == TipoJuego.GRAVITY){
				filas = line.getOptionValue("x");
			}
			else{
				System.err.println(MORE_PARAM);
				System.exit(1);
			}
		}
		SetupConfigurator opciones = new SetupConfigurator(juego, filas, columnas);
		return opciones;
		
	}
	
	private static SetupConfigurator optionSelector(String[] args){
		
		SetupConfigurator configuradorJuego = null;
		CommandLine line = null;
		CommandLineParser parser = new BasicParser();//Objeto creado para parsear args con las distintas opciones.
		Options option = new Options();//Opciones que se crean
		
		option.addOption("g", "game", true, "Tipo de juego (c4, co, gr). Por defecto, c4.");
		option.addOption("h", "help", false, "Muestra la ayuda sobre como utilizar los parametros.");
		option.addOption("y", "tamY", true, "Numero de columnas del tablero(solo para Gravity). Por defecto, 10.");
		option.addOption("x", "tamX", true, "Numero de filas del tablero(solo para Gravity). Por defecto, 10");
		
		try{
			line = parser.parse(option, args);//El metodo parse analiza los argumentos de acuerdo con las opciones especificadas.
			String[] argumento = line.getArgs();
			System.out.println(argumento.toString());
			
		}
		catch(ParseException exp){
			System.err.println(ERR_PARAM + exp.getMessage() + GET_HELP);//Si falla algo en el parseo saca el mensaje de error.
			System.exit(0);
		}
		
		if(line.hasOption("h")){
			showHelp();
			configuradorJuego = null;
			System.exit(0);
		}
		if(line.hasOption("g")){//Evaluo la opcion q ha metido el usuario despues de "g".
			String argumento = line.getOptionValue("g");//Obtengo el valor del argumento de esa opcion.
			//Tratamos ese argumento dependiendo del valor que tenga.
			if(argumento.equalsIgnoreCase("c4")){
				configuradorJuego = setupMode(line, TipoJuego.CONECTA4);
			}
			else if(argumento.equalsIgnoreCase("co")){
				configuradorJuego = setupMode(line, TipoJuego.COMPLICA);
			}
			else if(argumento.equalsIgnoreCase("gr")){
				configuradorJuego = setupMode(line, TipoJuego.GRAVITY);
			}
			else{
				System.err.println(NO_GAME + "'" + argumento + "'"+ " incorrecto" + GET_HELP);
				System.exit(1);
			}
		}
		return configuradorJuego;
		
	}
	
	private static void generaConecta4(){
		
		FactoriaJuegos factoria = new FactoriaJuegoConecta4();
		Partida partida = new Partida(factoria.creaReglas());
		Scanner sc = new Scanner(System.in);
		Controlador controlador = new Controlador(factoria, partida, sc);
		controlador.run();
		
	}
	
	private static void generaComplica(){
		
		FactoriaJuegos factoria = new FactoriaJuegoComplica();
		Partida partida = new Partida(factoria.creaReglas());
		Scanner sc = new Scanner(System.in);
		Controlador controlador = new Controlador(factoria, partida, sc);
		controlador.run();
		
	}
	
	private static void generaGravity(){
		
		FactoriaJuegos factoria = new FactoriaJuegoGravity();
		Partida partida = new Partida(factoria.creaReglas());
		Scanner sc = new Scanner(System.in);
		Controlador controlador = new Controlador(factoria, partida, sc);
		controlador.run();
		
	}
	
	public static void main(String[] args) {
		// TODO Auto-generated method stub
		SetupConfigurator setup;
		TipoJuego juego = TipoJuego.CONECTA4;
		
		ReglasJuego reglasGravity = new ReglasGravity();
		int filas = reglasGravity.getFilas();
		int columnas = reglasGravity.getColumnas();
		
		setup = optionSelector(args);
		if(setup != null){
			juego = setup.getJuego();
			if(setup.getNumFilas() != null){
				filas = Integer.parseInt(setup.getNumFilas());
			}
			if(setup.getNumColumnas() != null){
				columnas = Integer.parseInt(setup.getNumColumnas());
			}
		}
		if(juego == TipoJuego.CONECTA4){
			generaConecta4();
		}
		else if(juego == TipoJuego.COMPLICA){
			generaComplica();
		}
		else if(juego == TipoJuego.GRAVITY){
			generaGravity();
		}
		
	}
	
}
		
		
	
