package Model;

import java.util.Stack;
import Model.Enumerate.*;
import Model.Exceptions.*;
import Model.Movimientos.*;
import Model.Reglas.*;
import Model.Jugadores.*;
import Controladores.Factorias.*;

public class Partida {
	
	private Tablero tablero;
	private Ficha turno;
	private boolean terminada;
	private Ficha ganador;
	private boolean tablas;
	private Stack<Movimiento> movimientosStack;
	private ReglasJuego reglas;
	
	public static final int MAX_UNDO = 10;
	
	//Constructora.
	public Partida(ReglasJuego juegoElegido){
		
		this.reset(juegoElegido);
		
	}
	
	//Metodos accedentes y mutadores
	
	public Ficha getTurno(){
		
		return this.turno;
		
	}
	
	private void setTurno(Ficha color){
		
		this.turno = color;
		
	}
	
	public Ficha getGanador(){
		
		return this.ganador;
		
	}
	
	public void setGanador(Ficha color){
		
		this.ganador = color;
		
	}
	
	//Metodos
	public void reset(ReglasJuego reglasJuegoElegido){
		
		this.reglas = reglasJuegoElegido;
		this.tablero = reglasJuegoElegido.iniciaTablero();
		this.setTurno(reglasJuegoElegido.jugadorInicial());
		this.terminada = false;
		this.setGanador(Ficha.VACIA);
		this.movimientosStack = new Stack<Movimiento>();
		
	}
	
	public boolean partidaEnTablas(){
		
		return this.tablas;
		
	}
	
	public boolean partidaTerminada(){
		
		return this.terminada;
		
	}
	
	public Movimiento getMovimiento(FactoriaJuegos factoria, Jugador jugador) throws DatosIncorrectos{
		
		try{
			return jugador.getMovimiento(factoria, this.tablero, this.getTurno());
		}
		catch(DatosIncorrectos e){
			throw e;
		}
		
	}
	
	public void ejecutaMovimiento (Movimiento mov) throws MovimientoInvalido {
		
		if (this.terminada) {
			throw new MovimientoInvalido ("La partida ya esta terminada.");
			
		}
		Ficha color = mov.getTurno();
		if(color != this.getTurno()) {
			throw new MovimientoInvalido ("Turno incorrecto.");
			
		}
		//Realizamos el movimiento
		mov.ejecutaMovimiento(this.tablero);
		//Apilamos el movimiento en la pila de movimientos
		this.movimientosStack.push(mov);
		//Comprobamos si hay ganador o hay tablas en el caso del conecta 4 y gravity
		//y luego cambiamos el turno.
		this.setGanador(this.reglas.hayGanador(mov.getFila(), mov.getColumna(), mov.getTurno(), this.tablero));
		if(this.getGanador() != Ficha.VACIA){
			this.terminada = true;
		}
		else if(this.reglas.tablas(this.tablero)){
			this.terminada = true;
		}
		else{//Cambiamos el turno
			this.setTurno(this.reglas.siguienteTurno(this.getTurno()));
		}
		
	}
	
	public void undo() throws ErrorDeEjecucion{
		
		//Comprobamos si hay movimientos en la pila de movimientos
		//Si no hay movimientos lanzamos la excepcion que la tratara el undo del controlador
		if(this.movimientosStack.size() > 0){
			//Deshacemos el movimiento
			Movimiento ultimo = movimientosStack.pop();
			ultimo.undo(this.tablero);
			this.setTurno(ultimo.getTurno());
		}
		else{//Lanzamos la excepcion
			throw new ErrorDeEjecucion("No hay movimientos anteriores para deshacer.");
		}
		
	}
	
	public String indicarTurno() {
		
		String colorJugador = "";
		if (this.getTurno() == Ficha.BLANCA) {
			colorJugador = colorJugador + "Juegan Blancas!";
		} 
		else {
			colorJugador = colorJugador + "Juegan Negras!";
		}
		return colorJugador;

	}
	
	public String escribeGanador(){
		
		String ganador = "";
		if(this.getGanador() == Ficha.BLANCA){
			ganador = "BLANCAS";
		}
		else if(this.getGanador() == Ficha.NEGRA){
			ganador = "NEGRAS";
		}
		return ganador;
		
	}
	
	public String toString(){
		
		return this.tablero.toString();
		
	}

}
